#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <conio.h>
#include <alloc.h>
#include <process.h>

/*
 *  Utility:  XTNDPATH 1.1 (eXTeND MS-DOS environment PATH length -- FAST!)
 *
 *  Author:       G. David Douglas, Jr.  (douglas@usceast.cs.scarolina.edu)
 *  Date:          1/22/FR/93
 *  Compiler:     Turbo C 1.5 under MS-DOS 4.01, using Compact memory model
 *  DOS version:  4.01 or above
 *
 *  Usage:
 *
 *  XTNDPATH [-REV | -Q] [-PREFIX AlternatePrefixString]
 *
 *  or
 *
 *  XTNDPATH Anything-else           (for a help screen to be shown)
 *
 *  All items in brackets are optional and must be specified in the shown
 *  order; don't use brackets when giving optional items.  For example,
 *  use
 *
 *  XTNDPATH -REV
 *
 *  not
 *
 *  XTNDPATH [-REV]
 *
 *  XTNDPATH will extend the DOS PATH length by appending all environment
 *  entries that begin with PATHPART (the default PrefixString).
 *
 *  -REV and -Q are "mutually exclusive" parameters -- one or the other
 *  may be given, but not both.  if -REV is specified (MUST BE FIRST
 *  PARAMETER), XTNDPATH displays environment modification information
 *  in reverse video (black characters on white background); this
 *  "reverse video" output CANNOT be redirected, for instance, to
 *  \dev\nul.  if -Q is specified (MUST BE FIRST PARAMETER), XTNDPATH
 *  displays NO environment modification information, REGARDLESS of
 *  whether or not any environment areas were modified.  Otherwise,
 *  any environment modification information output is displayed with
 *  the current text attributes (this CAN be redirected to \dev\nul).
 *
 *  If -PREFIX AlternatePrefixString is specified, then the given
 *  AlternatePrefixString is used instead of the default PATHPART prefix.
 *
 *  Possible ways of giving the XTNDPATH command:
 *
 *
 *  XTNDPATH                          (to simply extend the DOS PATH length by
 *                                     appending all environment entries that
 *                                     begin with PATHPART.  Any environment
 *                                     modification information is displayed
 *                                     using current text attributes.)
 *
 *
 *  XTNDPATH -REV                     (to extend the DOS PATH in the same way
 *                                     as when XTNDPATH is used with no
 *                                     parameters, but display information
 *                                     output in black characters on white
 *                                     background.)
 *
 *  XTNDPATH -Q                       (to extend the DOS PATH in the same way
 *                                     as when XTNDPATH is used with no
 *                                     parameters, but display NO environment
 *                                     modification information.)
 *
 *  XTNDPATH -PREFIX AlternatePrefixString (to extend the DOS PATH length
 *                                          by appending all environment
 *                                          entries that begin with
 *                                          some AlternatePrefixString,
 *                                          e.g., PATHPIECE)
 *
 *  XTNDPATH -REV -PREFIX AlternatePrefixString  (To combine the last given
 *  or                                            case with the two cases
 *  XTNDPATH -Q -PREFIX AlternatePrefixString     before that.)
 *
 *  XTNDPATH anything-else                 (To display help text directly)
 *
 *  For instance,  XTNDPATH HELP
 *
 *
 *  This utility, when run, searches through ALL memory blocks allocated
 *  by MS-DOS for any in use by COMMAND.COM, and, for each COMMAND.COM
 *  environment block it finds, checks to see if (a) a PATH= entry exists
 *  there, and (b) if any entries of the form PATHPART= or PATHPART1=,
 *  PATHPART2=, PATHPARTQWERTY=, etc. exist.
 *
 *  The default prefix for appended entries is PATHPART; an alternate
 *  prefix can be used by giving the -PREFIX option followed by the
 *  alternate prefix on the command line.
 *
 *  If both conditions (a) and (b) are satisfied, it edits the PATH=
 *  statement, appending all PATHPARTn= commands to the end of the PATH=
 *  statement, in the order IN WHICH THEY ARE FOUND, and then displays a
 *  one-line message showing the location of the environment block modified
 *  along with the original and new PATH= lengths.  In other words, if
 *
 *  PATHPART2=C:\TEMP;C:\TEXTDIR;
 *  PATH=C:\;C:\DOS;
 *  PATHPARTQWERTY=C:\QWERTY;
 *
 *  are found, then regardless of the fact that PATHPART2 was found
 *  before PATHPARTqwerty, PATH= is modified to be
 *
 *  PATH=C:\;C:\DOS;C:\TEMP;C:\TEXTDIR;C:\QWERTY;
 *
 *
 *  The same result may be accomplished with
 *
 *  SET PATHPIECE2=C:\TEMP;C:\TEXTDIR;
 *  PATH=C:\;C:\DOS;
 *  SET PATHPIECEQWERTY=C:\QWERTY;
 *  XTNDPATH -PREFIX PATHPIECE
 *
 *
 *  Note that each original path component, that is, the PATH= component
 *  or each PATHPARTn= component, is considered a complete path in and
 *  of itself.  However, if there are two path components to be joined, and
 *  the first one does NOT end with a ';' (semicolon), one will be included
 *  to keep the resulting extended path valid.
 *
 *  The utility does not consider some characters valid for inclusion in a
 *  PATH statement; therefore, if any of these are found, they are ignored.
 *  See the definition of INVALID_PATH_CHARACTERS, found immediately after
 *  this header comment.
 *
 *  IMPORTANT:  It is possible, having run XTNDPATH, for the PATH= statement
 *  to exceed 255 characters.  If, under these conditions, after XTNDPATH
 *  is run, another PATH= statement is given on the command line, part of
 *  the previously extended PATH= statement may be ignored by COMMAND.COM,
 *  producing an extra, un-deletable, environment entry.  For this reason,
 *  XTNDPATH checks for COMMAND.COM environment entries which contain no
 *  '=' character, and deletes them.  This is safe because COMMAND.COM allows
 *  no environment entry to be created without the use of an '=' character.
 *  XTNDPATH will do this correction, WITH OR WITHOUT a PATH= entry and any
 *  PATHPARTn= entries in all found COMMAND.COM environment blocks.
 *
 *  Also note:  after the utility is done editing the PATH= statement,
 *  all PATHPARTn= statements are deleted, and are NOT present in the
 *  final modified environment.  The PATH= entry retains its original
 *  relative position in the modified environment's entries.
 *
 *  Final note:  for safety reasons, ONLY environment areas directly
 *  used by some copy of COMMAND.COM might be modified; all others are
 *  ignored.  This is because I cannot be sure that some utilities don't
 *  directly modify their environment areas.
 *
 *  This utility was inspired in part by the BIGPATH utility, written by
 *  Brian Moran, brian@mirror.TMC.COM  .  It has been tested successfully
 *  in extending the PATH= environment entry to over 1200 characters, after
 *  which the search for executables by COMMAND.COM is STILL successful!
 *
 *
 *  THIS SOURCE CODE IS ORIGINALLY (C)OPYRIGHT 1993 DAVID DOUGLAS,
 *  douglas@usceast.cs.scarolina.edu .  IT IS HEREBY PLACED IN THE PUBLIC
 *  DOMAIN AS FAR AS FURTHER USE AND MODIFICATION IS CONCERNED.  (THIS NOTICE
 *  MUST BE INCLUDED IN ANY MODIFIED VERSION OF THIS SOURCE CODE.)  USE THIS
 *  SOURCE CODE AT YOUR OWN RISK; ALL STANDARD DISCLAIMERS APPLY.  IT WORKS
 *  EXTREMELY WELL FOR ME; I HOPE YOU FIND IT AS USEFUL AS I HAVE.
 */

#define NOT_FINAL_MCB_CODE   0x4D                 /* 'M' */
#define FINAL_MCB_CODE       0x5A                 /* 'Z' */
#define BYTES_PER_PARAGRAPH  16

#define MAX_MCB_NAME_LEN     8

#define INVALID_PATH_CHARACTERS "*+|=,?\"<> []/"

#define DEFAULT_PREFIX_STRING "PATHPART"          /* IMPORTANT!  DON'T USE   */
                                                  /* ANY '=' CHARACTERS!!!   */

#define NORMAL_DISPLAY    0                       /* Constants used when     */
#define REV_VIDEO_DISPLAY 1                       /* determining how to      */
#define NO_DISPLAY        2                       /* display environment     */
                                                  /* modification            */
                                                  /* information output      */

#define NORMAL_EXIT_CODE           0              /* Possible error codes    */
#define NOTHING_MODIFIED_EXIT_CODE 1              /* to return when exiting  */
#define ERROR_EXIT_CODE            2              /* this utility, from best */
                                                  /* case to worst case.     */

typedef struct {
                unsigned char mcb_entry_code;     /* Last, or not last MCB?  */
                unsigned int  mcb_psp_segment;    /* Segment of Program-     */
                                                  /* Segment-Prefix with     */
                                                  /* which this Memory       */
                                                  /* Control Block is        */
                                                  /* associated              */

                unsigned int  mcb_size;           /* Size of Memory Control  */
                                                  /* Block area in segments  */
                                                  /* (16-byte paragraphs)    */

                unsigned char
                          unknown_mcb_info[3];    /* Three bytes in Memory   */
                                                  /* Control Block regarding */
                                                  /* whose purpose I have no */
                                                  /* information.            */

                unsigned char mcb_name
                          [MAX_MCB_NAME_LEN];     /* Possible program name   */
                                                  /* associated with Memory  */
                                                  /* Control Block           */

               } mem_control_block;   /* Definition for Memory Control Block */

